package com.zanxiang.sdk.service.pay;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.zanxiang.common.enums.HttpStatusEnum;
import com.zanxiang.common.enums.OrderStateEnum;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.exception.CustomException;
import com.zanxiang.common.utils.StringUtils;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.module.util.pojo.ResultVO;
import com.zanxiang.mybatis.entity.Order;
import com.zanxiang.sdk.constant.MiPayConstants;
import com.zanxiang.sdk.constant.RedisKeyConstant;
import com.zanxiang.sdk.domain.bo.ProductPayParamBO;
import com.zanxiang.sdk.domain.dto.*;
import com.zanxiang.sdk.service.IGameAppletService;
import com.zanxiang.sdk.service.IOrderService;
import com.zanxiang.sdk.service.IUserService;
import com.zanxiang.sdk.util.RedisUtil;
import com.zanxiang.sdk.util.WxPayUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.util.UriComponentsBuilder;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.net.URI;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * 米大师服务实现
 *
 * @author xufeng
 * @date 2022/7/14 16:33
 */
@Slf4j
@Service
public class MiPayService extends PayBaseService {

    @Autowired
    private IOrderService orderService;

    @Autowired
    private IGameAppletService gameAppletService;

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private IUserService userService;

    @Autowired
    private RedisUtil<String> redisUtil;

    @Value("${payConfig.miPay.isSand}")
    private Integer isSand;

    private MiPayConfigDTO config;

    @Override
    public Map<String, Object> create(ProductPayParamBO product, GamePayWayDTO gamePayWayDTO) {
        //查询订单
        PlatformOrderDTO orderInfo = orderService.getByOrderId(product.getOutTradeNo());
        if (Objects.isNull(orderInfo)) {
            throw new CustomException(HttpStatusEnum.ORDER_NO_FIND);
        }
        if (orderInfo.getGameId() == null || orderInfo.getGameId() == 0) {
            throw new CustomException(HttpStatusEnum.ORDER_GAME_ID_IS_NULL);
        }
        this.configInit(product.getGameId());
        //获取用户openId
        UserDTO userDTO = userService.getUserByUserId(product.getUserId());
        String openId = userDTO.getOpenId();
        //获取米大师钱包余额
        Long balance = this.midasGetBalance(openId);
        //返回参数
        Map<String, Object> payParamMap = new HashMap<>(9);
        //余额充足, 直接扣除
        if (balance >= orderInfo.getAmount().longValue()) {
            String billNo = this.midasPay(openId, orderInfo);
            payParamMap.put("status", "0");
            payParamMap.put("billNo", billNo);
            //支付成功
            this.paySuccess(product.getOutTradeNo(), orderInfo.getAmount().toString(), config.getAppId());
        } else {
            //余额不足, 返回订单id, 前端调充值接口, 充值完后再继续支付
            payParamMap.put("currencyType", "CNY");
            payParamMap.put("mode", "game");
            payParamMap.put("zoneId", "1");
            payParamMap.put("offerId", config.getAppId());
            payParamMap.put("env", String.valueOf(this.isSand));
            payParamMap.put("buyQuantity", orderInfo.getAmount().toString());
            payParamMap.put("status", "1");
            payParamMap.put("balance", String.valueOf(balance));
            payParamMap.put("orderId", product.getOutTradeNo());
            //更新订单商户信息
            orderService.update(new LambdaUpdateWrapper<Order>()
                    .set(Order::getMerchantNo, config.getAppId())
                    .set(Order::getStatus, OrderStateEnum.WAIT_PAY.getCode())
                    .eq(Order::getOrderId, product.getOutTradeNo()));
        }
        return payParamMap;
    }

    private Long midasGetBalance(String openId) {
        Map<String, String> paramMap = new HashMap<>();
        paramMap.put("openid", openId);
        paramMap.put("appid", config.getWxAppId());
        paramMap.put("offer_id", config.getAppId());
        paramMap.put("ts", String.valueOf(System.currentTimeMillis() / 1000));
        paramMap.put("zone_id", "1");
        paramMap.put("pf", "android");
        paramMap.put("sig", this.sign(MiPayConstants.BALANCE_URL, paramMap));
        Map<String, String> result = this.miPayApi(MiPayConstants.BALANCE_URL, paramMap);
        return Long.valueOf(result.get("balance"));
    }

    private String midasPay(String openId, PlatformOrderDTO orderInfo) {
        Map<String, String> paramMap = new HashMap<>();
        paramMap.put("appid", config.getWxAppId());
        paramMap.put("openid", openId);
        paramMap.put("offer_id", config.getAppId());
        paramMap.put("ts", String.valueOf(System.currentTimeMillis() / 1000));
        paramMap.put("pf", "android");
        paramMap.put("zone_id", "1");
        paramMap.put("amt", orderInfo.getAmount().toString());
        paramMap.put("bill_no", orderInfo.getOrderId());
        paramMap.put("sig", this.sign(MiPayConstants.PAY_URL, paramMap));
        Map<String, String> result = this.miPayApi(MiPayConstants.PAY_URL, paramMap);
        return result.get("bill_no");
    }

    private String getAccessToken(String appId, String appKey) {
        String redisKey = RedisKeyConstant.MI_PAY_TOKEN + appId;
        String cache = redisUtil.getCache(redisKey);
        if (StringUtils.isNotEmpty(cache)) {
            return cache;
        }
        //请求url
        URI uri = UriComponentsBuilder.fromHttpUrl(MiPayConstants.MI_PAY_HOST + MiPayConstants.TOKEN_URL)
                .queryParam("appid", appId)
                .queryParam("secret", appKey)
                .queryParam("grant_type", "client_credential")
                .build().toUri();
        String result;
        try {
            result = restTemplate.getForObject(uri, String.class);
        } catch (Exception e) {
            log.error("获取米大师token异常, appId : {}, appKey : {}, e : {}", appId, appKey, e.getMessage());
            throw new BaseException("获取米大师token异常");
        }
        if (Strings.isEmpty(result)) {
            log.error("获取米大师token失败, 返回结果为空, appId : {}, appKey : {}", appId, appKey);
            throw new BaseException("获取米大师token失败, 返回结果为空");
        }
        Map<String, String> data = JsonUtil.toMap(result, Map.class, String.class, String.class);
        String accessToken = data.get("access_token");
        String expiresIn = data.get("expires_in");
        if (Strings.isBlank(accessToken) || Strings.isBlank(expiresIn)) {
            log.error("获取米大师token失败, 返回结果参数不全, appId : {}, appKey : {}, data {}", appId, appKey, data);
            throw new BaseException("获取米大师token失败, 返回结果参数不全");
        }
        redisUtil.setCache(redisKey, accessToken, Long.valueOf(data.get("expires_in")));
        return accessToken;
    }

    private Map<String, String> miPayApi(String url, Map<String, String> paramMap) {
        //使用沙箱时，沙箱地址替换
        if (this.isSand == 1) {
            url = url.replace("/cgi-bin/midas/", "/cgi-bin/midas/sandbox/");
        }
        String appKey = this.isSand == 1 ? config.getAppKeyDev() : config.getAppKey();
        //接口凭证
        String accessToken = getAccessToken(config.getAppId(), appKey);
        //接口url
        URI uri = UriComponentsBuilder.fromHttpUrl(MiPayConstants.MI_PAY_HOST + url)
                .queryParam("access_token", accessToken).build().toUri();
        String result;
        try {
            result = restTemplate.postForObject(uri, paramMap, String.class);
        } catch (Exception e) {
            log.error("米大师接口调用失败, url : {}, paramMap : {}, e : {}", url, JsonUtil.toString(paramMap), e.getMessage());
            throw new BaseException("米大师接口调用失败");
        }
        if (StringUtils.isEmpty(result)) {
            log.error("米大师接口调用失败, 返回结果为空, url : {}, paramMap : {}", url, JsonUtil.toString(paramMap));
            throw new BaseException("米大师接口调用失败, 返回结果为空");
        }
        Map<String, String> data = JsonUtil.toMap(result, Map.class, String.class, String.class);
        //返回结果成功
        if (Objects.equals("0", data.get("errcode"))) {
            return data;
        }
        //返回结果失败
        log.error("米大师接口调用返回结果失败, url : {}, paramMap : {}, errcode : {}, errmsg : {}", url, paramMap,
                data.get("errcode"), data.get("errmsg"));
        throw new BaseException("米大师接口调用返回结果失败");
    }

    private void configInit(Long gameId) {
        //查询游戏小程序信息
        GameAppletDTO gameAppletDTO = gameAppletService.getByGameId(gameId);
        //支付配置赋值
        this.config = MiPayConfigDTO.builder()
                .wxAppId(gameAppletDTO.getAppId())
                .appId(gameAppletDTO.getMiPayAppId())
                .appKey(gameAppletDTO.getMiPayAppKey())
                .appKeyDev(gameAppletDTO.getMiPayAppKeyDev())
                .build();
    }

    private String sign(String url, Map<String, String> paramMap) {
        //沙箱环境地址替换
        if (this.isSand == 1) {
            url = url.replace("/cgi-bin/midas/", "/cgi-bin/midas/sandbox/");
        }
        String appKey = this.isSand == 1 ? config.getAppKeyDev() : config.getAppKey();
        //签名
        return WxPayUtil.miPaySin("POST", url, paramMap, appKey);
    }

    @Override
    public String notify(HttpServletRequest request, HttpServletResponse response) {
        throw new BaseException("米大师支付不存在异步回调");
    }

    @Override
    public ResultVO synNotify(HttpServletRequest request) {
        throw new BaseException("米大师支付不存在同步回调");
    }

    @Override
    public void closeOrder(PlatformOrderDTO platformOrderDTO) {
        throw new BaseException("米大师支付不存在取消订单");
    }
}
